export const TOGGLE_PANEL = 'TOGGLE_PANEL';
export const COLLAPSE_PANEL = 'COLLAPSE_PANEL';
export const SET_VALUE = 'SET_VALUE';
export const SUBMIT_FORM = 'SUBMIT_FORM';
export const REMOVE_VALUE = 'REMOVE_VALUE';
export const REMOVE_ALL_VALUES = 'REMOVE_ALL_VALUES';
export const CLEAR_SEARCH_INPUT = 'CLEAR_SEARCH_INPUT';
export const LINK = 'LINK';